﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using PerlinNoiseShader.PerlinNoise2D;
using PerlinNoiseShader.Simplex2D;
using PerlinNoiseShader.ImprovedNoise;

namespace PerlinNoiseShader.NoiseBlender
{
    public class NoiseBlenderClass
    {
        private int octaves = -1;
        private int octavesCos = -1;
        private int seed = -1;
        private WriteableBitmap[] aWriteableBitmap;
        private ImageBrush[] aImageBrush;
        private PerlinNoise2DClass oPerlinNoise2DClass = null;
        private Simplex2DClass oSimplex2DClass = null;
        private ImprovedNoiseClass oImprovedNoiseClass = null;
        private NoiseBlenderEffect oNoiseBlenderEffect = null;
        private NoiseBlenderCosineEffect oNoiseBlenderCosineEffect = null;
        private NoiseBlenderCosineEffect3D oNoiseBlenderCosineEffect3D = null;
        private UIElement iUIElement = null;
        private Transform oTransform=null;

        public int currentSeed
        {
            get { return seed; }
        }
        
        public UIElement customUIElement
        {
            get { return iUIElement; }
            set
            {
                iUIElement = value;
                if (oPerlinNoise2DClass != null)
                {
                    oPerlinNoise2DClass.customUIElement = iUIElement;
                }
                if (oSimplex2DClass != null)
                {
                    oSimplex2DClass.customUIElement = iUIElement;
                }
                if (oImprovedNoiseClass != null)
                {
                    oImprovedNoiseClass.customUIElement = iUIElement;
                }
            }
        }

        public Transform oCustomTransform
        {
            get { return oTransform; }
            set
            {
                oTransform = value;
                if (oPerlinNoise2DClass != null)
                {
                    oPerlinNoise2DClass.oCustomTransform = oTransform;
                }
                if (oSimplex2DClass != null)
                {
                    oSimplex2DClass.oCustomTransform = oTransform;
                }
                if (oImprovedNoiseClass != null)
                {
                    oImprovedNoiseClass.oCustomTransform = oTransform;
                }
            }
        }

        #region standard 2D Perlin noise
        public void noises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, bool absolute, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            noises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, absolute, tileable, c1, c2, oWriteableBitmap);
        }

        public void noises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, bool absolute, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            noises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, absolute, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }

        public void noises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            noises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, amplitude, xyMult, shift, tileable, c1, c2, oWriteableBitmap);
        }

        public void noises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            noises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, amplitude, xyMult, shift, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }
        
        public void noises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, bool absolute,bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if(aOffset.Length!=octaves) return;
            if(aAmplitude.Length!=octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                if (octaves > 1)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                else oNoiseBlenderEffect = null;

            }
            
                
            if (oPerlinNoise2DClass == null)
            {
                this.seed = seed;
                oPerlinNoise2DClass = new PerlinNoise2DClass(seed);
                oPerlinNoise2DClass.customUIElement = iUIElement;
                oPerlinNoise2DClass.oCustomTransform = oTransform;
            }else if (this.seed!=seed){
                this.seed = seed;
                oPerlinNoise2DClass.reInitialize(seed);
            }
            if (octaves == 1)
            {
                if (tileable)
                {
                    oPerlinNoise2DClass.singleTileableNoise(freqXY[0], aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);
                }
                else
                {
                    oPerlinNoise2DClass.singleNoise(freqXY[0], aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);
                }
            }
            else
            {
                if (oNoiseBlenderEffect.absolute != absolute)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                for (int i = 0; i < octaves; i++)
                {
                    if (tileable)
                    {
                        oPerlinNoise2DClass.tileableNoise(freqXY[i], aOffset[i], absolute, aWriteableBitmap[i]);
                    }
                    else
                    {
                        oPerlinNoise2DClass.noise(freqXY[i], aOffset[i], absolute, aWriteableBitmap[i]);
                    }
                }
                oPerlinNoise2DClass.currentUIElement.Effect = oNoiseBlenderEffect;
                oWriteableBitmap.Render(oPerlinNoise2DClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();
                
            }
        }

        public void noises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, bool absolute, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                if (octaves > 1)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                else oNoiseBlenderEffect = null;

            }


            if (oPerlinNoise2DClass == null)
            {
                this.seed = seed;
                oPerlinNoise2DClass = new PerlinNoise2DClass(seed);
                oPerlinNoise2DClass.customUIElement = iUIElement;
                oPerlinNoise2DClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oPerlinNoise2DClass.reInitialize(seed);
            }
            if (octaves == 1)
            {
                if (tileableX ||tileableY)
                {
                    oPerlinNoise2DClass.singleTileableNoise(freqXY[0], aOffset[0], aAmplitude[0], absolute,tileableX,tileableY, c1, c2, oWriteableBitmap);
                }
                else
                {
                    oPerlinNoise2DClass.singleNoise(freqXY[0], aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);
                }
            }
            else
            {
                if (oNoiseBlenderEffect.absolute != absolute)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                for (int i = 0; i < octaves; i++)
                {
                    if (tileableX || tileableY)
                    {
                        oPerlinNoise2DClass.tileableNoise(freqXY[i], aOffset[i], absolute, tileableX, tileableY, aWriteableBitmap[i]);
                    }
                    else
                    {
                        oPerlinNoise2DClass.noise(freqXY[i], aOffset[i], absolute, aWriteableBitmap[i]);
                    }
                }
                oPerlinNoise2DClass.currentUIElement.Effect = oNoiseBlenderEffect;
                oWriteableBitmap.Render(oPerlinNoise2DClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();

            }
        }

        public void noises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octavesCos)
            {
                this.octaves = octaves;
                oNoiseBlenderCosineEffect = new NoiseBlenderCosineEffect(octaves - 1);
                aWriteableBitmap = new WriteableBitmap[octaves];
                aImageBrush = new ImageBrush[octaves];
                for (int i = 0; i < octaves; i++)
                {
                    aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                    aImageBrush[i] = new ImageBrush();
                    aImageBrush[i].ImageSource = aWriteableBitmap[i];
                }
                oNoiseBlenderCosineEffect.aInput = aImageBrush;
                oNoiseBlenderCosineEffect.c1 = c1;
                oNoiseBlenderCosineEffect.c2 = c2;
                oNoiseBlenderCosineEffect.aAmplitude = aAmplitude;
                oNoiseBlenderCosineEffect.amplitude = 0.5f * amplitude;
                oNoiseBlenderCosineEffect.xyMult = new Point(xyMult.X * oWriteableBitmap.PixelWidth, xyMult.Y * oWriteableBitmap.PixelHeight);
                oNoiseBlenderCosineEffect.shift = shift;

            }


            if (oPerlinNoise2DClass == null)
            {
                this.seed = seed;
                oPerlinNoise2DClass = new PerlinNoise2DClass(seed);
                oPerlinNoise2DClass.customUIElement = iUIElement;
                oPerlinNoise2DClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oPerlinNoise2DClass.reInitialize(seed);
            }

            for (int i = 0; i < octaves; i++)
            {
                if (tileable)
                {
                    oPerlinNoise2DClass.tileableNoise(freqXY[i], aOffset[i], true, aWriteableBitmap[i]);
                }
                else
                {
                    oPerlinNoise2DClass.noise(freqXY[i], aOffset[i], true, aWriteableBitmap[i]);
                }
            }
            oPerlinNoise2DClass.currentUIElement.Effect = oNoiseBlenderCosineEffect;
            oWriteableBitmap.Render(oPerlinNoise2DClass.currentUIElement, oTransform);
            oWriteableBitmap.Invalidate();
        }

        public void noises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octavesCos)
            {
                this.octaves = octaves;
                oNoiseBlenderCosineEffect = new NoiseBlenderCosineEffect(octaves - 1);
                aWriteableBitmap = new WriteableBitmap[octaves];
                aImageBrush = new ImageBrush[octaves];
                for (int i = 0; i < octaves; i++)
                {
                    aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                    aImageBrush[i] = new ImageBrush();
                    aImageBrush[i].ImageSource = aWriteableBitmap[i];
                }
                oNoiseBlenderCosineEffect.aInput = aImageBrush;
                oNoiseBlenderCosineEffect.c1 = c1;
                oNoiseBlenderCosineEffect.c2 = c2;
                oNoiseBlenderCosineEffect.aAmplitude = aAmplitude;
                oNoiseBlenderCosineEffect.amplitude = 0.5f * amplitude;
                oNoiseBlenderCosineEffect.xyMult = new Point(xyMult.X*oWriteableBitmap.PixelWidth,xyMult.Y*oWriteableBitmap.PixelHeight);
                oNoiseBlenderCosineEffect.shift = shift;

            }


            if (oPerlinNoise2DClass == null)
            {
                this.seed = seed;
                oPerlinNoise2DClass = new PerlinNoise2DClass(seed);
                oPerlinNoise2DClass.customUIElement = iUIElement;
                oPerlinNoise2DClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oPerlinNoise2DClass.reInitialize(seed);
            }

            for (int i = 0; i < octaves; i++)
            {
                if (tileableX || tileableY)
                {
                    oPerlinNoise2DClass.tileableNoise(freqXY[i], aOffset[i], true, tileableX, tileableY, aWriteableBitmap[i]);
                }
                else
                {
                    oPerlinNoise2DClass.noise(freqXY[i], aOffset[i], true, aWriteableBitmap[i]);
                }
            }
            oPerlinNoise2DClass.currentUIElement.Effect = oNoiseBlenderCosineEffect;
            oWriteableBitmap.Render(oPerlinNoise2DClass.currentUIElement, oTransform);
            oWriteableBitmap.Invalidate();
        }

        public void turbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, bool absolute, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            turbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, absolute, tileable, c1, c2, oWriteableBitmap);
        }

        public void turbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, bool absolute, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            noises(aFreqXY, octaves, seed, aOffset, aAmplitudes, absolute, tileable, c1, c2, oWriteableBitmap);
        }
        public void turbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, bool absolute, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            turbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, absolute, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }

        public void turbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, bool absolute, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            noises(aFreqXY, octaves, seed, aOffset, aAmplitudes, absolute, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }

        public void turbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            turbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, amplitudeCos,xyMult,shift, tileable, c1, c2, oWriteableBitmap);
        }

        public void turbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, bool tileable, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            noises(aFreqXY, octaves, seed, aOffset, aAmplitudes, amplitudeCos,xyMult,shift, tileable, c1, c2, oWriteableBitmap);
        }
        public void turbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            turbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, amplitudeCos,xyMult,shift, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }

        public void turbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, bool tileableX, bool tileableY, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            noises(aFreqXY, octaves, seed, aOffset, aAmplitudes, amplitudeCos,xyMult,shift, tileableX, tileableY, c1, c2, oWriteableBitmap);
        }

        #endregion

        #region simplex 2D noise
        public void simplexNoises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            simplexNoises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, absolute, c1, c2, oWriteableBitmap);
        }

        public void simplexNoises(Size[] freqXY, int octaves, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            simplexNoises(freqXY, octaves, (new Random()).Next(), aOffset, aAmplitude, amplitude, xyMult, shift, c1, c2, oWriteableBitmap);
        }

        public void simplexNoises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                if (octaves > 1)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                else oNoiseBlenderEffect = null;

            }


            if (oSimplex2DClass == null)
            {
                this.seed = seed;
                oSimplex2DClass = new Simplex2DClass(seed);
                oSimplex2DClass.customUIElement = iUIElement;
                oSimplex2DClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oSimplex2DClass.reInitialize(seed);

            }
            if (octaves == 1)
            {
                oSimplex2DClass.singleNoise(freqXY[0], aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);
                
            }
            else
            {
                if (oNoiseBlenderEffect.absolute != absolute)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                for (int i = 0; i < octaves; i++)
                {
                    oSimplex2DClass.noise(freqXY[i], aOffset[i], absolute, aWriteableBitmap[i]);
                }
                oSimplex2DClass.currentUIElement.Effect = oNoiseBlenderEffect;
                oWriteableBitmap.Render(oSimplex2DClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();

            }
        }

        public void simplexNoises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (octaves != this.octavesCos)
            {
                this.octaves = octaves;
                oNoiseBlenderCosineEffect = new NoiseBlenderCosineEffect(octaves - 1);
                aWriteableBitmap = new WriteableBitmap[octaves];
                aImageBrush = new ImageBrush[octaves];
                for (int i = 0; i < octaves; i++)
                {
                    aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                    aImageBrush[i] = new ImageBrush();
                    aImageBrush[i].ImageSource = aWriteableBitmap[i];
                }
                oNoiseBlenderCosineEffect.aInput = aImageBrush;
                oNoiseBlenderCosineEffect.c1 = c1;
                oNoiseBlenderCosineEffect.c2 = c2;
                oNoiseBlenderCosineEffect.aAmplitude = aAmplitude;
                oNoiseBlenderCosineEffect.amplitude = 0.5f * amplitude;
                oNoiseBlenderCosineEffect.xyMult = new Point(xyMult.X * oWriteableBitmap.PixelWidth, xyMult.Y * oWriteableBitmap.PixelHeight);
                oNoiseBlenderCosineEffect.shift = shift;

            }


            if (oSimplex2DClass == null)
            {
                this.seed = seed;
                oSimplex2DClass = new Simplex2DClass(seed);
                oSimplex2DClass.customUIElement = iUIElement;
                oSimplex2DClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oSimplex2DClass.reInitialize(seed);

            }

            for (int i = 0; i < octaves; i++)
            {
                oSimplex2DClass.noise(freqXY[i], aOffset[i], true, aWriteableBitmap[i]);
            }
            oSimplex2DClass.currentUIElement.Effect = oNoiseBlenderCosineEffect;
            oWriteableBitmap.Render(oSimplex2DClass.currentUIElement, oTransform);
            oWriteableBitmap.Invalidate();
        }

        public void simplexTurbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            simplexTurbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, absolute, c1, c2, oWriteableBitmap);
        }

        public void simplexTurbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            simplexNoises(aFreqXY, octaves, seed, aOffset, aAmplitudes, absolute, c1, c2, oWriteableBitmap);
        }

        public void simplexTurbulence(Size freqXY, int octaves, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            simplexTurbulence(freqXY, octaves, (new Random()).Next(), aOffset, amplitude, persistence, amplitudeCos, xyMult, shift, c1, c2, oWriteableBitmap);
        }

        public void simplexTurbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            simplexNoises(aFreqXY, octaves, seed, aOffset, aAmplitudes, amplitudeCos, xyMult, shift, c1, c2, oWriteableBitmap);
        }
        #endregion

        #region flat improved noise
        public void improvedNoises(Size[] freqXY, int octaves, Point[] aOffset, float[] aZOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedNoises(freqXY, octaves, (new Random()).Next(), aOffset, aZOffset, aAmplitude, absolute, c1, c2, oWriteableBitmap);
        }
        public void improvedNoises(Size[] freqXY, int octaves, Point[] aOffset, float[] aZOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedNoises(freqXY, octaves, (new Random()).Next(), aOffset, aZOffset, aAmplitude, amplitude, xyMult, shift, c1, c2, oWriteableBitmap);
        }
        public void improvedNoises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aZOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (aZOffset.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                if (octaves > 1)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                else oNoiseBlenderEffect = null;

            }


            if (oImprovedNoiseClass == null)
            {
                this.seed = seed;
                oImprovedNoiseClass = new ImprovedNoiseClass(seed);
                oImprovedNoiseClass.customUIElement = iUIElement;
                oImprovedNoiseClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oImprovedNoiseClass.reInitialize(seed);

            }
            if (octaves == 1)
            {
                oImprovedNoiseClass.singleNoise(aZOffset[0], freqXY[0], aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);

            }
            else
            {
                if (oNoiseBlenderEffect.absolute != absolute)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                for (int i = 0; i < octaves; i++)
                {
                    oImprovedNoiseClass.noise(aZOffset[i], freqXY[i], aOffset[i], absolute, aWriteableBitmap[i]);
                }
                oImprovedNoiseClass.currentUIElement.Effect = oNoiseBlenderEffect;
                oWriteableBitmap.Render(oImprovedNoiseClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();

            }
        }

        public void improvedNoises(Size[] freqXY, int octaves, int seed, Point[] aOffset, float[] aZOffset, float[] aAmplitude, float amplitude, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (freqXY.Length != octaves) return;
            if (aZOffset.Length != octaves) return;
            if (octaves != this.octavesCos)
            {
                this.octaves = octaves;
                oNoiseBlenderCosineEffect = new NoiseBlenderCosineEffect(octaves - 1);
                aWriteableBitmap = new WriteableBitmap[octaves];
                aImageBrush = new ImageBrush[octaves];
                for (int i = 0; i < octaves; i++)
                {
                    aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                    aImageBrush[i] = new ImageBrush();
                    aImageBrush[i].ImageSource = aWriteableBitmap[i];
                }
                oNoiseBlenderCosineEffect.aInput = aImageBrush;
                oNoiseBlenderCosineEffect.c1 = c1;
                oNoiseBlenderCosineEffect.c2 = c2;
                oNoiseBlenderCosineEffect.aAmplitude = aAmplitude;
                oNoiseBlenderCosineEffect.amplitude = 0.5f * amplitude;
                oNoiseBlenderCosineEffect.xyMult = new Point(xyMult.X * oWriteableBitmap.PixelWidth, xyMult.Y * oWriteableBitmap.PixelHeight);
                oNoiseBlenderCosineEffect.shift = shift;
            }


            if (oImprovedNoiseClass == null)
            {
                this.seed = seed;
                oImprovedNoiseClass = new ImprovedNoiseClass(seed);
                oImprovedNoiseClass.customUIElement = iUIElement;
                oImprovedNoiseClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oImprovedNoiseClass.reInitialize(seed);

            }

            for (int i = 0; i < octaves; i++)
            {
                oImprovedNoiseClass.noise(aZOffset[i], freqXY[i], aOffset[i], true, aWriteableBitmap[i]);
            }
            oImprovedNoiseClass.currentUIElement.Effect = oNoiseBlenderCosineEffect;
            oWriteableBitmap.Render(oImprovedNoiseClass.currentUIElement, oTransform);
            oWriteableBitmap.Invalidate();


        }


        public void improvedTurbulence(Size freqXY, int octaves, Point[] aOffset, float[] aZOffset,float amplitude, float persistence, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedTurbulence(freqXY, octaves, (new Random()).Next(), aOffset, aZOffset, amplitude, persistence, absolute, c1, c2, oWriteableBitmap);
        }

        public void improvedTurbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float[] aZOffset, float amplitude, float persistence, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            if (aZOffset == null) aZOffset = new float[octaves];

            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            improvedNoises(aFreqXY, octaves, seed, aOffset, aZOffset, aAmplitudes, absolute, c1, c2, oWriteableBitmap);
        }

        public void improvedTurbulence(Size freqXY, int octaves, Point[] aOffset, float[] aZOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedTurbulence(freqXY, octaves, (new Random()).Next(), aOffset, aZOffset, amplitude, persistence, amplitudeCos,xyMult,shift, c1, c2, oWriteableBitmap);
        }

        public void improvedTurbulence(Size freqXY, int octaves, int seed, Point[] aOffset, float[] aZOffset, float amplitude, float persistence, float amplitudeCos, Point xyMult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Point[octaves];
            if (aZOffset == null) aZOffset = new float[octaves];

            Size[] aFreqXY = new Size[octaves];
            float[] aAmplitudes = new float[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Point(0, 0);
                aFreqXY[i] = new Size(freqXY.Width, freqXY.Height);
                freqXY.Width *= 2.0;
                freqXY.Height *= 2.0;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            improvedNoises(aFreqXY, octaves, seed, aOffset, aZOffset, aAmplitudes, amplitudeCos,xyMult,shift, c1, c2, oWriteableBitmap);
        }
        #endregion
        
        #region 3D improved noise
        
        public void improvedNoises(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, Vector3d[] aMult, Vector3d[] aOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedNoises(xPlane, yPlane, zPlane, octaves, (new Random()).Next(), aMult, aOffset, aAmplitude, absolute, c1, c2, oWriteableBitmap);
        }

        public void improvedNoises(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, int seed, Vector3d[] aMult,Vector3d[] aOffset, float[] aAmplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (octaves > 1 && aMult.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                if (octaves > 1)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                else oNoiseBlenderEffect = null;

            }


            if (oImprovedNoiseClass == null)
            {
                this.seed = seed;
                oImprovedNoiseClass = new ImprovedNoiseClass(seed);
                oImprovedNoiseClass.customUIElement = iUIElement;
                oImprovedNoiseClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oImprovedNoiseClass.reInitialize(seed);

            }
            if (octaves == 1)
            {
                oImprovedNoiseClass.singleNoise(xPlane, yPlane, zPlane, aOffset[0], aAmplitude[0], absolute, c1, c2, oWriteableBitmap);

            }
            else
            {
                if (oNoiseBlenderEffect.absolute != absolute)
                {
                    oNoiseBlenderEffect = new NoiseBlenderEffect(octaves - 1, absolute);
                    oNoiseBlenderEffect.aInput = aImageBrush;
                    oNoiseBlenderEffect.c1 = c1;
                    oNoiseBlenderEffect.c2 = c2;
                    oNoiseBlenderEffect.aAmplitude = aAmplitude;
                }
                for (int i = 0; i < octaves; i++)
                {
                    oImprovedNoiseClass.noise(xPlane, yPlane, zPlane,aMult[i], aOffset[i], absolute, aWriteableBitmap[i]);
                }
                oImprovedNoiseClass.currentUIElement.Effect = oNoiseBlenderEffect;
                oWriteableBitmap.Render(oImprovedNoiseClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();

            }
        }

        public void improvedNoises(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, Vector3d [] aMult,Vector3d[] aOffset, float[] aAmplitude, float amplitude,Vector3d mult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedNoises(xPlane, yPlane, zPlane, octaves, (new Random()).Next(), aMult, aOffset, aAmplitude, amplitude, mult, shift, c1, c2, oWriteableBitmap);
        }
        
        public void improvedNoises(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, int seed, Vector3d[] aMult,Vector3d[] aOffset, float[] aAmplitude, float amplitude, Vector3d mult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset.Length != octaves) return;
            if (aAmplitude.Length != octaves) return;
            if (octaves > 1 && aMult.Length != octaves) return;
            if (octaves != this.octaves)
            {
                this.octaves = octaves;
                
                    oNoiseBlenderCosineEffect3D = new NoiseBlenderCosineEffect3D(octaves - 1);
                    aWriteableBitmap = new WriteableBitmap[octaves];
                    aImageBrush = new ImageBrush[octaves];
                    for (int i = 0; i < octaves; i++)
                    {
                        aWriteableBitmap[i] = new WriteableBitmap(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
                        aImageBrush[i] = new ImageBrush();
                        aImageBrush[i].ImageSource = aWriteableBitmap[i];
                    }
                    oNoiseBlenderCosineEffect3D.xPlane = xPlane;
                    oNoiseBlenderCosineEffect3D.yPlane = yPlane;
                    oNoiseBlenderCosineEffect3D.zPlane = zPlane;
                    oNoiseBlenderCosineEffect3D.offsetXY = new Point(aOffset[0].x,aOffset[0].y);
                    oNoiseBlenderCosineEffect3D.offsetZ = (float)aOffset[0].z;
                    oNoiseBlenderCosineEffect3D.multXY = new Point(aMult[0].x,aMult[0].y);
                    oNoiseBlenderCosineEffect3D.multZ = (float)aMult[0].z;
                    oNoiseBlenderCosineEffect3D.aInput = aImageBrush;
                    oNoiseBlenderCosineEffect3D.c1 = c1;
                    oNoiseBlenderCosineEffect3D.c2 = c2;
                    oNoiseBlenderCosineEffect3D.aAmplitude = aAmplitude;
                    oNoiseBlenderCosineEffect3D.amplitude = 0.5f * amplitude;
                    oNoiseBlenderCosineEffect3D.xyMult = new Point(mult.x,mult.y);
                    oNoiseBlenderCosineEffect3D.zMult = (float)mult.z;
                    oNoiseBlenderCosineEffect3D.shift = shift;
                

            }


            if (oImprovedNoiseClass == null)
            {
                this.seed = seed;
                oImprovedNoiseClass = new ImprovedNoiseClass(seed);
                oImprovedNoiseClass.customUIElement = iUIElement;
                oImprovedNoiseClass.oCustomTransform = oTransform;
            }
            else if (this.seed != seed)
            {
                this.seed = seed;
                oImprovedNoiseClass.reInitialize(seed);

            }
            
                
                for (int i = 0; i < octaves; i++)
                {
                    oImprovedNoiseClass.noise(xPlane, yPlane, zPlane, aMult[i], aOffset[i], true, aWriteableBitmap[i]);
                }
                oImprovedNoiseClass.currentUIElement.Effect = oNoiseBlenderCosineEffect3D;
                oWriteableBitmap.Render(oImprovedNoiseClass.currentUIElement, oTransform);
                oWriteableBitmap.Invalidate();

            
        }

        public void improvedTurbulence(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, Vector3d[] aOffset, float amplitude, float persistence, float amplitudeCos, Vector3d mult, float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            improvedTurbulence(xPlane, yPlane, zPlane, octaves, aOffset, amplitude, persistence, amplitudeCos, mult, shift, c1, c2, oWriteableBitmap);
        }

        public void improvedTurbulence(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, int octaves, int seed, Vector3d[] aOffset, float amplitude, float persistence, float amplitudeCos,Vector3d mult ,float shift, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            if (octaves <= 0 || octaves > 16) return;
            if (aOffset == null) aOffset = new Vector3d[octaves];
            
            float[] aAmplitudes = new float[octaves];
            float value = 1;
            Vector3d[] aMult = new Vector3d[octaves];
            for (int i = 0; i < octaves; i++)
            {
                if (aOffset[i] == null) aOffset[i] = new Vector3d(5000, 5000,5000);
                aMult[i] = new Vector3d(value,value,value);
                value *= 2f;
                aAmplitudes[i] = amplitude;
                amplitude *= persistence;
            }
            improvedNoises(xPlane,yPlane,zPlane,octaves,seed, aMult, aOffset,aAmplitudes, amplitudeCos, mult,shift, c1, c2, oWriteableBitmap);
        }


        #endregion
    }
}
