﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.NoiseBlender
{
    public class NoiseBlenderCosineEffect3D:ShaderEffect
    {
        private int index = -1;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        private DependencyProperty xPlaneProperty;
        private DependencyProperty yPlaneProperty;
        private DependencyProperty zPlaneProperty;
        private DependencyProperty[] aInputProperty;
        private DependencyProperty[] aAmplitudeProperty;
        private DependencyProperty amplitudeProperty;
        private DependencyProperty offsetXYProperty;
        private DependencyProperty offsetZProperty;
        private DependencyProperty multXYProperty;
        private DependencyProperty multZProperty;
        private DependencyProperty xyMultProperty;
        private DependencyProperty zMultProperty;
        private DependencyProperty shiftProperty;

        public NoiseBlenderCosineEffect3D(int index)
        {
            this.index = index;
            Uri u = new Uri(@"/PerlinNoiseShader;component/NoiseBlender/blend"+index.ToString()+"Cos3D.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            xPlaneProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("xArray", typeof(NoiseBlenderCosineEffect3D), 0);
            yPlaneProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("yArray", typeof(NoiseBlenderCosineEffect3D), 1);
            zPlaneProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("zArray", typeof(NoiseBlenderCosineEffect3D), 2);
            UpdateShaderValue(xPlaneProperty);
            UpdateShaderValue(yPlaneProperty);
            UpdateShaderValue(zPlaneProperty);
            offsetXYProperty = DependencyProperty.Register("offsetXY", typeof(Point), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(new Point(), PixelShaderConstantCallback(0)));
            offsetZProperty = DependencyProperty.Register("offsetZ", typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(0f, PixelShaderConstantCallback(1)));
            multXYProperty = DependencyProperty.Register("multXY", typeof(Point), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(new Point(1, 1), PixelShaderConstantCallback(2)));
            multZProperty = DependencyProperty.Register("multZ", typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(3)));
            c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(4)));
            c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(5)));
            amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(6)));
            xyMultProperty = DependencyProperty.Register("xyMult", typeof(Point), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(7)));
            zMultProperty = DependencyProperty.Register("zMult", typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(8)));
            shiftProperty = DependencyProperty.Register("shift", typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(0f, PixelShaderConstantCallback(9)));
            UpdateShaderValue(offsetXYProperty);
            UpdateShaderValue(offsetZProperty);
            UpdateShaderValue(multXYProperty);
            UpdateShaderValue(multZProperty);
            UpdateShaderValue(c1Property);
            UpdateShaderValue(c2Property);
            UpdateShaderValue(amplitudeProperty);
            UpdateShaderValue(xyMultProperty);
            UpdateShaderValue(zMultProperty);
            UpdateShaderValue(shiftProperty);
            
            aInputProperty = new DependencyProperty[index + 1];
            aAmplitudeProperty = new DependencyProperty[index/2 + 1];
            for (int i = 0; i < aInputProperty.Length; i++)
            {
                aInputProperty[i] = ShaderEffect.RegisterPixelShaderSamplerProperty("input" + (i + 1).ToString(), typeof(NoiseBlenderCosineEffect3D), i+3);
                UpdateShaderValue(aInputProperty[i]);
            }
            for (int i = 0; i < (index+1) / 2; i++)
            {
                aAmplitudeProperty[i] = DependencyProperty.Register("amplitude" + (i + 1).ToString(), typeof(Point), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(new Point(1,1), PixelShaderConstantCallback(i + 10)));
                UpdateShaderValue(aAmplitudeProperty[i]);
            }
            if ((index & 1) == 0)
            {
                aAmplitudeProperty[index/2] = DependencyProperty.Register("amplitude" + (index+1).ToString(), typeof(float), typeof(NoiseBlenderCosineEffect3D), new PropertyMetadata(1f, PixelShaderConstantCallback(index+10)));
                UpdateShaderValue(aAmplitudeProperty[index/2]);
            }
            
        }

        public Brush xPlane
        {
            get { return (Brush)GetValue(xPlaneProperty); }
            set { SetValue(xPlaneProperty, value); }
        }

        public Brush yPlane
        {
            get { return (Brush)GetValue(yPlaneProperty); }
            set { SetValue(yPlaneProperty, value); }
        }

        public Brush zPlane
        {
            get { return (Brush)GetValue(zPlaneProperty); }
            set { SetValue(zPlaneProperty, value); }
        }
        
        public Point offsetXY
        {
            get { return (Point)GetValue(offsetXYProperty); }
            set { SetValue(offsetXYProperty, value); }
        }

        public float offsetZ
        {
            get { return (float)GetValue(offsetZProperty); }
            set { SetValue(offsetZProperty, value); }
        }

        public Point multXY
        {
            get { return (Point)GetValue(multXYProperty); }
            set { SetValue(multXYProperty, value); }
        }

        public float multZ
        {
            get { return (float)GetValue(multZProperty); }
            set { SetValue(multZProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public Brush[] aInput
        {
            get
            {
                Brush[] aBrush = new Brush[index + 1];
                for (int i = 0; i <= index; i++)
                {
                    aBrush[i] = (Brush)GetValue(aInputProperty[i]);
                }
                return aBrush;
            }
            set
            {
                Brush[] aBrush = (Brush[])value;
                for (int i = 0; i <= index; i++)
                {
                    SetValue(aInputProperty[i], aBrush[i]);
                }
            }
        }

        public float[] aAmplitude
        {
            get
            {
                float[] aFloat = new float[index + 1];
                for (int i = 0; i < (index+1)/2; i++)
                {
                    Point amplitude=(Point)GetValue(aAmplitudeProperty[i]);
                    aFloat[2 * i] = (float)amplitude.X;
                    aFloat[2 * i + 1] = (float)amplitude.Y;
                }
                if ((index & 1) == 0)
                {
                    aFloat[index] = (float)GetValue(aAmplitudeProperty[index/2]);
                }
                return aFloat;
            }
            set
            {
                float[] aFloat = (float[])value;
                for (int i = 0; i < (index+1)/2; i++)
                {
                    Point amplitude = new Point(aFloat[2*i],aFloat[2*i+1]);
                    SetValue(aAmplitudeProperty[i], amplitude);
                }
                if ((index & 1) == 0)
                {
                    float amplitude = aFloat[index];
                    SetValue(aAmplitudeProperty[index/2], amplitude);
                }
            }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Point xyMult
        {
            get { return (Point)GetValue(xyMultProperty); }
            set { SetValue(xyMultProperty, value); }
        }

        public float zMult
        {
            get { return (float)GetValue(zMultProperty); }
            set { SetValue(zMultProperty, value); }
        }

        public float shift
        {
            get { return (float)GetValue(shiftProperty); }
            set { SetValue(shiftProperty, value); }
        }
    }
}
