﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.ColorMap
{
    /// <summary>
    /// the red, green, blue and alpha component of a source image refer to color maps. the values of the color maps are beeing added
    /// </summary>
    public class ColorMapEffect : ShaderEffect
    {
        private DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(ColorMapEffect), 0);
        private DependencyProperty[] aColorMapProperty;
        private DependencyProperty sizeProperty, size2Property;
        private ImageBrush[] aColorMapBrush;
        private WriteableBitmap sourceBitmap;
        private ImageBrush oSourceBrush = new ImageBrush();
        private int count = 0;
        private WriteableBitmap[] aColorMapBitmap;
        private Uri u;

        public WriteableBitmap sourceImage
        {
            get { return sourceBitmap; }
            set
            {
                sourceBitmap = value;
                oSourceBrush.ImageSource = sourceBitmap;
                if (u != null) SetValue(sourceImageProperty, oSourceBrush);
            }
        }

        /// <summary>
        /// 1..4 color maps, if there are less than 4 color maps, the other components of the source image are not used
        /// </summary>
        public WriteableBitmap[] aColorBitmap
        {
            get { return aColorMapBitmap; }
            set
            {
                aColorMapBitmap = value;
                if (aColorMapBitmap == null)
                {
                    count = 0;
                    u = null;
                }
                else
                {
                    if (aColorMapBitmap.Length != count)
                    {
                        count = aColorMapBitmap.Length;
                        if (count > 0 && count <= 4)
                        {
                            u = new Uri(@"/PerlinNoiseShader;component/ColorMap/ColorMapShader" + count.ToString() + ".ps", UriKind.Relative);
                            this.PixelShader = new PixelShader() { UriSource = u };
                            sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(ColorMapEffect), 0);
                            UpdateShaderValue(sourceImageProperty);
                            SetValue(sourceImageProperty, oSourceBrush);
                            aColorMapProperty = new DependencyProperty[count];
                            aColorMapBrush = new ImageBrush[count];
                            for (int i = 1; i <= count; i++)
                            {
                                aColorMapProperty[i - 1] = ShaderEffect.RegisterPixelShaderSamplerProperty("colorMap" + ((i > 1) ? i.ToString() : ""), typeof(ColorMapEffect), i);
                                UpdateShaderValue(aColorMapProperty[i - 1]);
                                aColorMapBrush[i - 1] = new ImageBrush();
                                aColorMapBrush[i - 1].ImageSource = aColorMapBitmap[i - 1];
                                SetValue(aColorMapProperty[i - 1], aColorMapBrush[i - 1]);
                            }
                            switch (count)
                            {
                                case 1:
                                    {
                                        sizeProperty = DependencyProperty.Register("size", typeof(float), typeof(ColorMapEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(0)));
                                        UpdateShaderValue(sizeProperty);
                                        SetValue(sizeProperty, 255f / (float)aColorMapBitmap[0].PixelWidth);
                                        break;
                                    }
                                case 2:
                                    {
                                        sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(ColorMapEffect), new PropertyMetadata(new Size(), PixelShaderConstantCallback(0)));
                                        UpdateShaderValue(sizeProperty);
                                        SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                        break;
                                    }
                                case 3:
                                    {
                                        sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(ColorMapEffect), new PropertyMetadata(new Size(), PixelShaderConstantCallback(0)));
                                        size2Property = DependencyProperty.Register("size2", typeof(float), typeof(ColorMapEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(1)));
                                        UpdateShaderValue(sizeProperty);
                                        UpdateShaderValue(size2Property);
                                        SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                        SetValue(size2Property, 255f / (float)aColorMapBitmap[2].PixelWidth);
                                        break;
                                    }
                                case 4:
                                    {
                                        sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(ColorMapEffect), new PropertyMetadata(new Size(), PixelShaderConstantCallback(0)));
                                        size2Property = DependencyProperty.Register("size2", typeof(Size), typeof(ColorMapEffect), new PropertyMetadata(new Size(), PixelShaderConstantCallback(1)));
                                        UpdateShaderValue(sizeProperty);
                                        UpdateShaderValue(size2Property);
                                        SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                        SetValue(size2Property, new Size(255.0 / aColorMapBitmap[2].PixelWidth, 255.0 / aColorMapBitmap[3].PixelWidth));
                                        break;
                                    }
                            }
                        }
                    }
                    else if (count > 0)
                    {
                        for (int i = 1; i <= count; i++)
                        {
                            aColorMapBrush[i - 1].ImageSource = aColorMapBitmap[i - 1];
                            SetValue(aColorMapProperty[i - 1], aColorMapBrush[i - 1]);
                        }
                        switch (count)
                        {
                            case 1:
                                {
                                    SetValue(sizeProperty, 255f / (float)aColorMapBitmap[0].PixelWidth);
                                    break;
                                }
                            case 2:
                                {
                                    SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                    break;
                                }
                            case 3:
                                {
                                    SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                    SetValue(size2Property, 255f / (float)aColorMapBitmap[2].PixelWidth);
                                    break;
                                }
                            case 4:
                                {
                                    SetValue(sizeProperty, new Size(255.0 / aColorMapBitmap[0].PixelWidth, 255.0 / aColorMapBitmap[1].PixelWidth));
                                    SetValue(size2Property, new Size(255.0 / aColorMapBitmap[2].PixelWidth, 255.0 / aColorMapBitmap[3].PixelWidth));
                                    break;
                                }
                        }
                    }
                }

            }
        }
    }
}
