﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.ColorMap
{
    /// <summary>
    /// the red and green components of a source bitmap refer to a two dimensional color map
    /// </summary>
    public class ColorMapEffect2D:ShaderEffect
    {
        public static readonly DependencyProperty sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(ColorMapEffect2D), new PropertyMetadata(new Size(255.0 / 256.0, 255.0 / 256.0), PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(ColorMapEffect2D), 0);
        public static readonly DependencyProperty colorMapProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("colorMap", typeof(ColorMapEffect2D), 1);
        private ImageBrush sourceImageBrush = new ImageBrush(), colorMapBrush = new ImageBrush();
        private WriteableBitmap sourceBitmap, colorMapBitmap;
        public ColorMapEffect2D()
        {
            Uri u = new Uri(@"/PerlinNoiseShader;component/ColorMap/ColorMapShader2D.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(sizeProperty);
            UpdateShaderValue(sourceImageProperty);
            UpdateShaderValue(colorMapProperty);
        }

        public WriteableBitmap sourceImage
        {
            get { return sourceBitmap; }
            set
            {
                sourceBitmap = value;
                sourceImageBrush.ImageSource = sourceBitmap;
                SetValue(sourceImageProperty, sourceImageBrush);
            }
        }

        public WriteableBitmap colorMap
        {
            get { return colorMapBitmap; }
            set
            {
                colorMapBitmap = value;
                colorMapBrush.ImageSource = colorMapBitmap;
                SetValue(colorMapProperty, colorMapBrush);
            }
        }

        public Size size
        {
            get
            {
                Size oSize = new Size();
                Size thisSize = (Size)GetValue(sizeProperty);
                oSize.Width = 1.0 / thisSize.Width / 255.0;
                oSize.Height = 1.0 / thisSize.Height / 255.0;
                return oSize;
            }


            set
            {
                Size oSize = new Size();
                oSize.Width = 255.0 / value.Width;
                oSize.Height = 255.0 / value.Height;
                SetValue(sizeProperty, oSize);
            }
        }
    }
}
