﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.DisplacementMapFilter
{
    /// <summary>
    /// uses the red (x axis) and green (y axis) component of a discplacement bitmap to displace the pixels of a source bitmap
    /// </summary>
    public class DisplacementMapEffect:ShaderEffect
    {
        public static readonly DependencyProperty scaleXYProperty = DependencyProperty.Register("scaleXY", typeof(Size), typeof(DisplacementMapEffect), new PropertyMetadata(new Size(1, 1), PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(DisplacementMapEffect), 0);
        public static readonly DependencyProperty displacementImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("displacementImage", typeof(DisplacementMapEffect), 1);
        private WriteableBitmap sourceBitmap, displacementBitmap;
        private Size oScaleXY = new Size(1, 1);
        /// <summary>
        /// Initialization
        /// </summary>
        /// <param name="abs">displacement in + or - direction (false) or only in + direction (true)</param>
        public DisplacementMapEffect(bool abs)
        {
            Uri u;
            if(abs) 
                 u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/AbsDisplacementShader.ps", UriKind.Relative);
            else
                 u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/DisplacementShader.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(scaleXYProperty);
            UpdateShaderValue(sourceImageProperty);
            UpdateShaderValue(displacementImageProperty);
        }

        public Size scaleXY
        {
            get { return (Size)GetValue(scaleXYProperty); }
            set
            {
                oScaleXY = value;
                if (sourceBitmap != null)
                {
                    Size currentScaleXY = new Size();
                    //convert scale to normalized texture coordinates
                    currentScaleXY.Width = 255.0 * oScaleXY.Width / sourceBitmap.PixelWidth;
                    currentScaleXY.Height = 255.0 * oScaleXY.Height / sourceBitmap.PixelHeight;
                    SetValue(scaleXYProperty, currentScaleXY);
                }
            }
        }

        public WriteableBitmap sourceImage
        {
            get { return sourceBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                sourceBitmap = value;
                oImageBrush.ImageSource = sourceBitmap;
                SetValue(sourceImageProperty, oImageBrush);
                Size currentScaleXY = new Size();
                //convert scale to normalized texture coordinates
                currentScaleXY.Width = 255.0 * oScaleXY.Width / sourceBitmap.PixelWidth;
                currentScaleXY.Height = 255.0 * oScaleXY.Height / sourceBitmap.PixelHeight;
                SetValue(scaleXYProperty, currentScaleXY);    
            }
        }

        public WriteableBitmap displacementImage
        {
            get { return displacementBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                displacementBitmap=value;
                oImageBrush.ImageSource = displacementBitmap;
                SetValue(displacementImageProperty, oImageBrush);
            }
        }


    }
}
