﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.DisplacementMapFilter
{
    
    /// <summary>
    /// uses the red component of a displacement image to displace the pixels of a source bitmap along a radius of a circle
    /// </summary>
    public class RadialDisplacementEffect:ShaderEffect
    {
        public static readonly DependencyProperty scaleProperty = DependencyProperty.Register("scale", typeof(float), typeof(RadialDisplacementEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty subLengthProperty=DependencyProperty.Register("subLength",typeof(float),typeof(RadialDisplacementEffect),new PropertyMetadata(0f,PixelShaderConstantCallback(1)));
        public static readonly DependencyProperty scaleLengtProperty = DependencyProperty.Register("scaleLength", typeof(float), typeof(RadialDisplacementEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
        public static readonly DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(RadialDisplacementEffect), 0);
        public static readonly DependencyProperty displacementImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("displacementImage", typeof(RadialDisplacementEffect), 1);
        private WriteableBitmap sourceBitmap, displacementBitmap;
        private float fScale = 1;
        /// <summary>
        /// Initialization
        /// </summary>
        /// <param name="abs">displacement in + or - direction (false) or only in + direction (true) from the center</param>
        public RadialDisplacementEffect(bool abs)
        {
            Uri u;
            if(abs)
                u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/AbsRadialDisplacementShader.ps", UriKind.Relative);
            else
                u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/RadialDisplacementShader.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(scaleProperty);
            UpdateShaderValue(subLengthProperty);
            UpdateShaderValue(scaleLengtProperty);
            UpdateShaderValue(sourceImageProperty);
            UpdateShaderValue(displacementImageProperty);
        }

        public float scale
        {
            get { return (float)GetValue(scaleProperty); }
            set
            {
                fScale = value;
                if (sourceBitmap != null)
                {
                    //convert scale to normalized texture coordinates
                    float currentScale = 255.0f * fScale / (float)sourceBitmap.PixelWidth;
                    SetValue(scaleProperty, currentScale);
                }
            }
        }

        public float subLength
        {
            get { return (float)GetValue(subLengthProperty); }
            set { SetValue(subLengthProperty, value); }
        }

        public float scaleLength
        {
            get { return (float)GetValue(scaleLengtProperty); }
            set { SetValue(scaleLengtProperty, value); }
        }

        public WriteableBitmap sourceImage
        {
            get { return sourceBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                sourceBitmap = value;
                oImageBrush.ImageSource = sourceBitmap;
                SetValue(sourceImageProperty, oImageBrush);
                float currentScale = 255.0f * fScale / (float)sourceBitmap.PixelWidth;
                SetValue(scaleProperty, currentScale);
            }
        }

        public WriteableBitmap displacementImage
        {
            get { return displacementBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                displacementBitmap = value;
                oImageBrush.ImageSource = displacementBitmap;
                SetValue(displacementImageProperty, oImageBrush);
            }
        }
    }
}
