﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.DisplacementMapFilter
{
    /// <summary>
    /// uses the red component (x axis) of a displacement bitmap to displace the pixels of a source bitmap
    /// </summary>
    public class XDisplacementMapEffect:ShaderEffect
    {
        public static readonly DependencyProperty scaleProperty = DependencyProperty.Register("scale", typeof(float), typeof(XDisplacementMapEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(XDisplacementMapEffect), 0);
        public static readonly DependencyProperty displacementImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("displacementImage", typeof(XDisplacementMapEffect), 1);
        private WriteableBitmap sourceBitmap, displacementBitmap;
        private float fScale = 1;
        public XDisplacementMapEffect(bool abs)
        {
            Uri u;
            if (abs)
                u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/1DAbsDisplacementShader2.ps", UriKind.Relative);
            else
                u = new Uri(@"/PerlinNoiseShader;component/DisplacementMapFilter/1DDisplacementShader2.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(scaleProperty);
            UpdateShaderValue(sourceImageProperty);
            UpdateShaderValue(displacementImageProperty);
        }
        public float scale
        {
            get { return (float)GetValue(scaleProperty); }
            set
            {
                fScale = value;
                if (sourceBitmap != null)
                {
                    //convert scale to normalized texture coordinates
                    float currentScale = 255.0f * fScale / (float)sourceBitmap.PixelWidth;
                    SetValue(scaleProperty, currentScale);
                }
            }
        }

        public WriteableBitmap sourceImage
        {
            get { return sourceBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                sourceBitmap = value;
                oImageBrush.ImageSource = sourceBitmap;
                SetValue(sourceImageProperty, oImageBrush);
                //convert scale to normalized texture coordinates
                float currentScale = 255.0f * fScale / (float)sourceBitmap.PixelWidth;
                SetValue(scaleProperty, currentScale);
            }
        }

        public WriteableBitmap displacementImage
        {
            get { return displacementBitmap; }
            set
            {
                ImageBrush oImageBrush = new ImageBrush();
                displacementBitmap = value;
                oImageBrush.ImageSource = displacementBitmap;
                SetValue(displacementImageProperty, oImageBrush);
            }
        }
    }
}
