﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.NoiseBlender
{
    public class NoiseBlenderCosineEffect:ShaderEffect
    {
        private int index = -1;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        private DependencyProperty[] aInputProperty;
        private DependencyProperty[] aAmplitudeProperty;
        private DependencyProperty amplitudeProperty;
        private DependencyProperty xyMultProperty;
        private DependencyProperty shiftProperty;

        public NoiseBlenderCosineEffect(int index)
        {
            this.index = index;
            Uri u = new Uri(@"/PerlinNoiseShader;component/NoiseBlender/blend"+index.ToString()+"Cos.ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(0)));
            c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(1)));
            UpdateShaderValue(c1Property);
            UpdateShaderValue(c2Property);
            aInputProperty = new DependencyProperty[index + 1];
            aAmplitudeProperty = new DependencyProperty[index + 1];
            for (int i = 0; i < aInputProperty.Length; i++)
            {
                aInputProperty[i] = ShaderEffect.RegisterPixelShaderSamplerProperty("input" + (i + 1).ToString(), typeof(NoiseBlenderCosineEffect), i);
                UpdateShaderValue(aInputProperty[i]);
                aAmplitudeProperty[i] = DependencyProperty.Register("amplitude" + (i + 1).ToString(), typeof(float), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(i + 2)));
                UpdateShaderValue(aAmplitudeProperty[i]);
            }
            amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(index + 3)));
            UpdateShaderValue(amplitudeProperty);
            xyMultProperty = DependencyProperty.Register("xyMult", typeof(Point), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(index + 4)));
            UpdateShaderValue(xyMultProperty);
            shiftProperty = DependencyProperty.Register("shift", typeof(float), typeof(NoiseBlenderCosineEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(index + 5)));
            UpdateShaderValue(shiftProperty);
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public Brush[] aInput
        {
            get
            {
                Brush[] aBrush = new Brush[index + 1];
                for (int i = 0; i <= index; i++)
                {
                    aBrush[i] = (Brush)GetValue(aInputProperty[i]);
                }
                return aBrush;
            }
            set
            {
                Brush[] aBrush = (Brush[])value;
                for (int i = 0; i <= index; i++)
                {
                    SetValue(aInputProperty[i], aBrush[i]);
                }
            }
        }

        public float[] aAmplitude
        {
            get
            {
                float[] aFloat = new float[index + 1];
                for (int i = 0; i <= index; i++)
                {
                    aFloat[i] = (float)GetValue(aAmplitudeProperty[i]);
                }
                return aFloat;
            }
            set
            {
                float[] aFloat = (float[])value;
                for (int i = 0; i <= index; i++)
                {
                    float amplitude = aFloat[i];
                    SetValue(aAmplitudeProperty[i], amplitude);
                }
            }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Point xyMult
        {
            get { return (Point)GetValue(xyMultProperty); }
            set { SetValue(xyMultProperty, value); }
        }

        public float shift
        {
            get { return (float)GetValue(shiftProperty); }
            set { SetValue(shiftProperty, value); }
        }
    }
}
