﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.NoiseBlender
{
    public class NoiseBlenderEffect:ShaderEffect
    {
        private int index = 1;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        private DependencyProperty[] aInputProperty;
        private DependencyProperty[] aAmplitudeProperty;
        public readonly bool absolute;
        

        public NoiseBlenderEffect(int index, bool abs)
        {
            absolute = abs;
            this.index = index;
            string sAbs = abs ? "Abs" : "";
            Uri u = new Uri(@"/PerlinNoiseShader;component/NoiseBlender/blend" + index.ToString() + sAbs + ".ps", UriKind.Relative);
            this.PixelShader = new PixelShader() { UriSource = u };
            c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(NoiseBlenderEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(0)));
            c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(NoiseBlenderEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(1)));
            UpdateShaderValue(c1Property);
            UpdateShaderValue(c2Property);
            aInputProperty = new DependencyProperty[index+1];
            aAmplitudeProperty = new DependencyProperty[index+1];
            for (int i = 0; i < aInputProperty.Length; i++)
            {
                aInputProperty[i] = ShaderEffect.RegisterPixelShaderSamplerProperty("input" + (i + 1).ToString(), typeof(NoiseBlenderEffect), i);
                UpdateShaderValue(aInputProperty[i]);
                aAmplitudeProperty[i] = DependencyProperty.Register("amplitude" + (i + 1).ToString(), typeof(float), typeof(NoiseBlenderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(i + 2)));
                UpdateShaderValue(aAmplitudeProperty[i]);
            }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public Brush[] aInput
        {
            get { 
                Brush[] aBrush = new Brush[index+1];
                for (int i = 0; i <= index; i++)
                {
                     aBrush[i] = (Brush)GetValue(aInputProperty[i]);
                }
                return aBrush;
            }
            set
            {
                Brush[] aBrush = (Brush[])value;
                for (int i = 0; i <= index; i++)
                {
                    SetValue(aInputProperty[i], aBrush[i]);
                }
            }
        }

        public float[] aAmplitude
        {
            get
            {
                float[] aFloat = new float[index+1];
                for (int i = 0; i <= index; i++)
                {
                    aFloat[i] = absolute ? (float)GetValue(aAmplitudeProperty[i]) : (float)GetValue(aAmplitudeProperty[i]) * 2f;
                }
                return aFloat;
            }
            set
            {
                float[] aFloat = (float[])value;
                for (int i = 0; i <= index; i++)
                {
                    float amplitude=absolute?aFloat[i]:aFloat[i] * 0.5f;
                    SetValue(aAmplitudeProperty[i], amplitude);
                }
            }
        }


    }
}
