﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.RadialMapper
{
    /// <summary>
    /// maps an image to a ring
    /// </summary>
    public class RadialMapperEffect:ShaderEffect
    {
        public readonly DependencyProperty sourceImageProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("sourceImage", typeof(RadialMapperEffect), 0);
        public readonly DependencyProperty addLengthProperty = DependencyProperty.Register("addLength", typeof(float), typeof(RadialMapperEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(0)));
        public readonly DependencyProperty scaleProperty = DependencyProperty.Register("scale", typeof(float), typeof(RadialMapperEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(1)));
        private WriteableBitmap oBitmap = null;

        public RadialMapperEffect()
        {
            Uri u = new Uri(@"/PerlinNoiseShader;component/RadialMapper/RadialMapper.ps", UriKind.Relative);
            PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(sourceImageProperty);
            UpdateShaderValue(addLengthProperty);
            UpdateShaderValue(scaleProperty);
        }

        public WriteableBitmap sourceImage
        {
            get { return oBitmap; }
            set
            {
                oBitmap = value;
                ImageBrush oImageBrush = new ImageBrush();
                oImageBrush.ImageSource = oBitmap;
                SetValue(sourceImageProperty, oImageBrush);
            }
        }
        
        /// <summary>
        /// distance to the center
        /// </summary>
        public float addLength
        {
            get { return (float)GetValue(addLengthProperty); }
            set { SetValue(addLengthProperty, value); }
        }

        public float scale
        {
            get { return (float)GetValue(scaleProperty); }
            set { SetValue(scaleProperty, value); }
        }
    }
}
