﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using PerlinNoiseShader.NoiseBlender;
using System.Windows.Media.Media3D;
using PerlinNoiseShader.ImprovedNoise2D;
using VoxelSpacing.Shader;
using PerlinNoiseShader;

namespace VoxelSpacing
{
    public partial class MainPage : UserControl
    {
        private const int WIDTH = 600;
        private const int HEIGHT = 400;
        private const int DEPTH = 400;

        WriteableBitmap oWriteableBitmap, geoMap, illumMap, shadowMap;
        private double angle = 90;
        private double rad = 90 * Math.PI / 180;
        private double xp = 512, yp = 800;
        private int hp = 10, vp = -70;//-60;
        private int whichMap = 0;
        private int color = 2;
        EventHandler animation;
        int step = 0;
        private double[] sinFov = new double[WIDTH], cosFov = new double[WIDTH];
                
        public MainPage()
        {
            InitializeComponent();
            this.KeyDown+=new KeyEventHandler(MainPage_KeyDown);
            animation=new EventHandler(CompositionTarget_Rendering);
            CompositionTarget.Rendering += animation;
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            if (step == 0)
            {
                this.label1.Content = "Calculating landscape...";
                step++;
            }else  if (step == 1)
            {
                //calculate landscape
                geoMap = new WriteableBitmap(1024, 1024);
                NoiseBlenderClass oNoiseBlenderClass = new NoiseBlenderClass();
                oNoiseBlenderClass.turbulence2D(new Size(8.0 / 1024.0, 8.0 / 1024.0), new iSize(8, 8), 7, /*random,*/ null, 1, 0.5f, false, Color.FromArgb(0, 0, 0, 0), Color.FromArgb(255, 255, 255, 255), geoMap);
                this.label1.Content = "Calculating illumination...";
                step++;
            }
            else if (step == 2)
            {
                //calculate the light map
                illumMap = new WriteableBitmap(1024, 1024);
                Rectangle illumRectangle = new Rectangle();
                illumRectangle.Width = 1024;
                illumRectangle.Height = 1024;
                illumRectangle.Fill = new SolidColorBrush(Colors.Red);
                IllumShader oIllumShader = new IllumShader();
                oIllumShader.geoMap = geoMap;
                illumRectangle.Effect = oIllumShader;
                illumMap.Render(illumRectangle, null);
                //illumination replaced by a shader
                /*
                int index = 0; 
                Vector3d light=new Vector3d(2,0,-1);
                light=light.normalize();
                for (int y = 0; y < 1024; y++)
                {
                    for (int x = 0; x < 1024; x++)
                    {
                        int z1 = geoMap.Pixels[1024 * y + x] & 255;
                        int z2 = geoMap.Pixels[1024 * y + ((x - 1 + 1024) & 1023)] & 255;
                        int z3 = geoMap.Pixels[1024 * ((y - 1+1024) & 1023) + x] & 255;
                        Vector3d v1 = new Vector3d(1, 0, z1-z2);
                        Vector3d v2 = new Vector3d(0, 1, z1-z3);
                    
                        v1 = v1.normalize();
                        v2 = v2.normalize();
                        Vector3d v3 = v2.cross(v1);
                        v3 = v3.normalize();
                        double result = v3.dot(light);
                        if (result < 0) result = 0;
                        if (result > 1) result = 1;
                        int color = (int)(0.5*(255 * result+z1));
                        illumMap.Pixels[index] = z1 << 24 | color << 16 | color << 8 | color;
                        index++;
                    }
                }
                */

                illumMap.Invalidate();
                this.label1.Content = "Calculating shadow map...";
                step++;
            }
            else if (step == 3)
            {
                //calculate the shadow
                shadowMap = new WriteableBitmap(1024, 1024);
                int index = 0;
                //for every pixel at the bitmap: follow back the light vector. if there is an object before reaching the maximum height,
                //there must be a shadow for the current pixel
                for (int y = 0; y < 1024; y++)
                {
                    for (int x = 0; x < 1024; x++)
                    {
                        int z1 = geoMap.Pixels[index] & 255;
                        int color = illumMap.Pixels[index] & 255;
                        double height = z1;
                        int index2 = 1;
                        while (height < 255)
                        {
                            height += 2;
                            int z2 = geoMap.Pixels[1024 * y + ((x - index2 + 1024) & 1023)] & 255;
                            if (height <= z2)
                            {
                                color = 120 * z1 / 255 / 3;
                                break;
                            }
                            index2++;
                        }
                        shadowMap.Pixels[index] = z1 << 24 | color << 16 | color << 8 | color;
                        index++;
                    }
                }
                shadowMap.Invalidate();
                oWriteableBitmap = new WriteableBitmap(WIDTH, HEIGHT);
                this.label1.Content = "";
                this.label1.Visibility=Visibility.Collapsed;
                initTables();
                drawLandscape();
                myImage.Source = oWriteableBitmap;
                step++;
            }else if (step == 4) CompositionTarget.Rendering -= animation;
            
        }

        private void initTables()
        {
            double fov = 90;
            double fovHalf = fov / 2;
            double fovStart = Math.Tan(fovHalf * Math.PI / 180) * DEPTH;
            double x3d = -fovStart;
            double dx = (fovStart * 2) / (WIDTH - 1);
            for (int i = 0; i < WIDTH; i++, x3d += dx)
            {
                double atan = Math.Atan2(x3d, DEPTH);
                sinFov[i] = Math.Sin(atan);
                cosFov[i] = Math.Cos(atan);
                
            }
        }
        //floating point version
        /*
        private void drawLandscape()
        {
            
            Array.Clear(oWriteableBitmap.Pixels, 0, oWriteableBitmap.Pixels.Length);
            int screenAt = 150;//(int)((WIDTH)/2.0/Math.tan(fovHalf*Math.PI/180));
            WriteableBitmap readMap;
            if (whichMap == 0)
            {
                readMap = geoMap;
            }
            else if (whichMap == 1)
            {
                readMap = illumMap;
            }
            else
            {
                readMap = shadowMap;
            }
            int selectColor;
            if (color == 0)
            {
                selectColor = 0xFF0000;
            }
            else if (color==1)
            {
                selectColor = 0xFF00;
            }
            else{
                selectColor = 0xFF;
            } 
            try
            {
                int start = (hp) * screenAt / (HEIGHT - 1 + vp);
                if (start < 1) start = 1;
                double sin = Math.Sin(-rad);
                double cos = Math.Cos(-rad);
                double negCosAngle = Math.Cos(rad);
                double negSinAngle = Math.Sin(rad);
                for (int i = 0; i < WIDTH; i++)
                {
                    double rotx = cosFov[i] * cos - sinFov[i] * sin;//Math.cos(Math.atan2(x3d,DEPTH)+angle);
                    double roty = sinFov[i] * cos + cosFov[i] * sin;//Math.sin(Math.atan2(x3d,DEPTH)+angle);
                    double x = xp + rotx*start;
                    double y = yp + roty*start;

                    double xv = ((int)x) - xp;
                    double yv = ((int)y) - yp;
                    double length = xv*negCosAngle-yv*negSinAngle;
                    while (((int)length) <= 1)
                    {
                        x += rotx;
                        y += roty;
                        xv = ((int)x) - xp;
                        yv = ((int)y) - yp;
                        length = xv * negCosAngle - yv * negSinAngle;
                    }
                    int ymin = HEIGHT;
                    while ((int)length < DEPTH)
                    {
                        int index = ((int)y & 1023) * 1024 + ((int)x & 1023);
                        int pixel, h;
                        pixel = readMap.Pixels[index];
                        //the height is stored in the alpha value and rgb contains the color
                        h = (pixel >> 24) & 255;
                        //select color
                        pixel = (pixel & selectColor) | 255 << 24;
                        //fog
                        if ((int)length > DEPTH - 255)
                        {
                            pixel = pixel & 0xFFFFFF;
                            pixel = (DEPTH - (int)length) << 24 | pixel;
                        }

                        h = 255 - h + hp;
                        int z3 = (int)(screenAt * h / length) - vp;

                        int offset = (ymin - 1) * WIDTH + i;
                        for (int k = ymin - 1; k >= z3 && k >= 0; k--)
                        {
                            oWriteableBitmap.Pixels[offset] = pixel;
                            offset -= WIDTH;
                        }
                        if (ymin > z3)
                        {
                            ymin = z3;
                        }

                        if (ymin < 0) break;
                        x += rotx;
                        y += roty;
                        xv = ((int)x) - xp;
                        yv = ((int)y) - yp;
                        length = xv * negCosAngle - yv * negSinAngle;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace);
            }
            oWriteableBitmap.Invalidate();
        }
        */

        //fixed point version, much faster
        private void drawLandscape()
        {

            Array.Clear(oWriteableBitmap.Pixels, 0, oWriteableBitmap.Pixels.Length);
            int screenAt = 150;//(int)((WIDTH)/2.0/Math.tan(fovHalf*Math.PI/180));
            int screenAtShift = screenAt << 8;
            WriteableBitmap readMap;
            if (whichMap == 0)
            {
                readMap = geoMap;
            }
            else if (whichMap == 1)
            {
                readMap = illumMap;
            }
            else
            {
                readMap = shadowMap;
            }
            int selectColor;
            if (color == 0)
            {
                selectColor = 0xFF0000;
            }
            else if (color==1)
            {
                selectColor = 0xFF00;
            }
            else{
                selectColor = 0xFF;
            }
            try
            {
                int start = (hp) * screenAt / (HEIGHT - 1 + vp);
                if (start < 1) start = 1;
                double sin = Math.Sin(-rad);
                double cos = Math.Cos(-rad);
                double negCosAngle = Math.Cos(rad);
                double negSinAngle = Math.Sin(rad);
                for (int i = 0; i < WIDTH; i++)
                {
                    double rotx = cosFov[i] * cos - sinFov[i] * sin;//Math.cos(Math.atan2(x3d,DEPTH)+angle);
                    double roty = sinFov[i] * cos + cosFov[i] * sin;//Math.sin(Math.atan2(x3d,DEPTH)+angle);
                    double x = xp + rotx * start;
                    double y = yp + roty * start;

                    //16.16 fixed point arithemtic
                    int iLength = 0, cLength = 0;
                    int ix = (int)(65536 * x);
                    int iy = (int)(65536 * y);
                    int irotx = (int)(65536 * rotx);
                    int iroty = (int)(65536 * roty);
                    int ixp = (int)(65536 * xp);
                    int iyp = (int)(65536 * yp);
                    int ixv = 0, iyv = 0;
                    //22.10 fixed point arithmetic
                    int inegCosAngle = (int)(1024 * negCosAngle);
                    int inegSinAngle = (int)(1024 * negSinAngle);
                    ixv = (ix & (0xFFFF << 16) )- ixp;
                    iyv = (iy & (0xFFFF << 16)) - iyp;
                    iLength = ((ixv >> 6) * inegCosAngle - (iyv >> 6) * inegSinAngle) >> 4;
                    cLength = iLength >> 16;
                    while ((cLength) < 1)
                    {
                        ix += irotx;
                        iy += iroty;
                        ixv = (ix & (0xFFFF << 16)) - ixp;
                        iyv = (iy & (0xFFFF << 16)) - iyp;
                        iLength = ((ixv >> 6) * inegCosAngle - (iyv >> 6) * inegSinAngle) >> 4;
                        cLength = iLength >> 16;
                    }
                    int ymin = HEIGHT;
                    
                    while (cLength < DEPTH)
                    {
                        int index = ((iy>>16) & 1023) * 1024 + ((ix>>16) & 1023);
                        int pixel, h;
                        pixel = readMap.Pixels[index];
                        //the height is stored in the alpha value and rgb contains the color
                        h = (pixel >> 24) & 255;
                        //select color
                        pixel = (pixel & selectColor) | 255 << 24;
                        //fog
                        if (cLength > DEPTH - 255)
                        {
                            pixel = pixel & 0xFFFFFF;
                            pixel = (DEPTH - cLength) << 24 | pixel;
                        }

                        h = 255 - h + hp;
                        int z3 = screenAtShift * h / (iLength >> 8) - vp;

                        int offset = (ymin - 1) * WIDTH + i;
                        for (int k = ymin - 1; k >= z3 && k >= 0; k--)
                        {
                            oWriteableBitmap.Pixels[offset] = pixel;
                            offset -= WIDTH;
                        }
                        if (ymin > z3)
                        {
                            ymin = z3;
                        }

                        if (ymin < 0) break;
                        ix += irotx;
                        iy += iroty;
                        ixv = (ix & (0xFFFF << 16)) - ixp;
                        iyv = (iy & (0xFFFF << 16)) - iyp;
                        iLength = ((ixv >> 6) * inegCosAngle - (iyv >> 6) * inegSinAngle) >> 4;
                        cLength = iLength >> 16;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace);
            }
            oWriteableBitmap.Invalidate();
        }

        void MainPage_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Up:
                    {
                        xp += 2 * Math.Cos(rad);
                        yp -= 2 * Math.Sin(rad);
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Down:
                    {
                        xp -= 2 * Math.Cos(rad);
                        yp += 2 * Math.Sin(rad);
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Left:
                    {
                        angle += 5;
                        rad = angle * Math.PI / 180;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Right:
                    {
                        angle -= 5;
                        rad = angle * Math.PI / 180;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Q:
                    {
                        //move up
                        hp += 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.A:
                    {
                        //move down
                        hp -= 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.W:
                    {
                        //pseudo rotation
                        vp += 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.S:
                    {
                        //pseudo rotation
                        vp -= 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.Space:
                    {
                        //change the maps: normal, lighted, lighted + shadow
                        whichMap++;
                        if (whichMap > 2) whichMap = 0;
                        drawLandscape();
                        e.Handled = true;
                        break;
                    }
                case Key.R:
                    {
                        //red
                        color = 0;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.G:
                    {
                        //green
                        color = 1;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }
                case Key.B:
                    {
                        //blue
                        color = 2;
                        e.Handled = true;
                        drawLandscape();
                        break;
                    }    
                
            }
        }
        
    }
}
